
#include "HexGridDisplay.h"
#include "HexDisplay.h"
#include "const7.h"
#include "RealmColorMap.h"

HexGridDisplay::HexGridDisplay( bool invert) {
	inverted = invert;
	BackgroundColors = nullptr;
	FillEnabled = false;
	LineColor = RGBColor(255, 255, 255);
	gridsize = 1;
	hexradius = 48;
	centerhex.setx(0);
	centerhex.sety(0);
}

/* draw the hex grid onto SDLRenderer */
void HexGridDisplay::draw(SDL_Renderer* renderer) {

	// SDL_SetRenderDrawColor(renderer, linecolor->getred(), linecolor->getgreen(), linecolor->getblue(), SDL_ALPHA_OPAQUE);


	HexDisplay HexOutputter;

	// Pixel position of center of current hex...
	int hexcenterx, hexcentery;

	// start at center, render outward
	hexcenterx = center.getx();
	hexcentery = center.gety();

	// current radial distance (in hex units) from center of grid
	int hexdistance = 0;

	// index of hexes being drawn
	int hexindex = 0;

	// how many since the hexdistance radius inceremented?
	int cycleidx = 0;

	// circumference starts at 0, expands
	int circumference = 0;

	// current angle to step for next hex to render... 0 = east
	int stepdirection = 0;

	// cyclephase... side of the hexgrid currently being processed
	int cyclephase = 0;

	// steps between each cyclephase... increases as the size grows
	int phasesteps = 1;

	bool donedrawing = false;
	bool endphase = false;
	bool finished_cycle = true;

	// index within the current cyclephase
	int phaseidx = 0;

	// position of currently rendering hex (in h-draw increment units)
	int hexposx = 0;
	int hexposy = 0;




	int hexidx = 0;

	HexOutputter.setLineColor(LineColor);

	HexOutputter.setRadius(hexradius);

	if (FillEnabled) {
		HexOutputter.setFillColor(FillColor);
		HexOutputter.setFillEnabled(FillEnabled);
	}


	if (!inverted) {
		hexcenterx = center.getx() + ( (hexposx - centerhex.getx()) * hexradius * 1.5);
		hexcentery = center.gety() + ( (hexposy - centerhex.gety()) * hexradius * SQRT_3 / 2);
	} else {
		hexcenterx = center.getx() + ( (hexposx - centerhex.getx() ) * hexradius * SQRT_3 / 2);
		hexcentery = center.gety() + ( (hexposy - centerhex.gety() ) * hexradius * 1.5);
	}



	// loop until we reach the edge of the screen or our limit
	while (!donedrawing) {
		if (BackgroundColors) {
			HexOutputter.setFillColor(BackgroundColors->getHex(hexidx++));
		}

		if (!inverted) {
			HexOutputter.DrawHex(renderer,
					hexcenterx,
					hexcentery
					);
		} else {
				HexOutputter.DrawInvertedHex(renderer,
					hexcenterx,
					hexcentery
					);

		}


			// update indexes,
			// update direction...
			hexindex++;
			cycleidx++;
			phaseidx++;

			// check if it's time to end a phase
			if (phaseidx >= phasesteps) {
				endphase = true;
			}

			if (endphase) {

				cyclephase++;
				if (cyclephase > 5) {
					// when we finish the last phase of the cycle, flag...
					cyclephase = 0;
					finished_cycle = true;
				}

				// set the direction to begin the next phase
				if (cyclephase == 0) {
					stepdirection = 1;
					phasesteps = hexdistance;
				} else if (cyclephase == 1) {
					stepdirection = 2;
					phasesteps = hexdistance;
				} else if (cyclephase == 2) {
					stepdirection = 3;
					phasesteps = hexdistance;
				} else if (cyclephase == 3) {
					stepdirection = 4;
					phasesteps = hexdistance;
				} else if (cyclephase == 4) {
					stepdirection = 5;
					phasesteps = hexdistance;
				} else if (cyclephase == 5) {
					stepdirection = 0;
					if (hexdistance < gridsize )
						phasesteps = hexdistance + 1;
					else
						phasesteps = hexdistance;
				}

				// reset for next phase loop
				phaseidx = 0;
				endphase = false;
			}

			if (finished_cycle) {
				hexdistance++;
				circumference += 6;
				cycleidx = 0;
				cyclephase = 0;

				if (hexdistance <= 1)
					stepdirection = 0;
				else
					stepdirection = 1;

				if (hexdistance > gridsize) {
						donedrawing = true;
				}

				finished_cycle = false;
			}

			// move to the next one...
			if (!inverted) {
				switch (stepdirection) {
					case 0: // semi-northeast
						hexposx += 1;
						hexposy -= 1;
						break;

					case 1: // north
						hexposy -= 2;
						break;

					case 2: // semi-northwest
						hexposx--;
						hexposy--;
						break;

					case 3: // semi-southwest
						hexposx--;
						hexposy++;
						break;

					case 4: // south
						hexposy += 2;
						break;

					case 5: // semi-southeast
						hexposx++;
						hexposy++;
						break;
				};

				hexcenterx = center.getx() + ( (hexposx - centerhex.getx()) * hexradius * 1.5);
				hexcentery = center.gety() + ( (hexposy - centerhex.gety()) * hexradius * SQRT_3 / 2);

			} else { // inverted
				switch (stepdirection) {
					case 0: // east
						hexposx += 2;
						break;

					case 1: // northeast
						hexposx ++;
						hexposy --;
						break;

					case 2: // northwest
						hexposx--;
						hexposy--;
						break;

					case 3: // west
						hexposx -= 2;
						break;

					case 4: // southwest
						hexposx --;
						hexposy ++;
						break;

					case 5: // southeast
						hexposx ++;
						hexposy ++;
						break;
				};

				hexcenterx = center.getx() + ( (hexposx - centerhex.getx() ) * hexradius * SQRT_3 / 2);
				hexcentery = center.gety() + ( (hexposy - centerhex.gety() ) * hexradius * 1.5);

			}
	}
}


