#include "RealmCharacterObjectPositioner.h"
#include "RealmSceneView.h"

using namespace std;

RealmCharacterObjectPositioner::RealmCharacterObjectPositioner(RealmCharacterGroupPositioner* group_positioner,
									RealmSceneView* scene_view, HexSpiralCoordinate start_scene_coordinate,
									RealmSceneView* zone_view, HexSpiralCoordinate start_zone_coordinate,
									RealmSceneView* region_view, HexSpiralCoordinate start_region_coordinate
									):
		SceneAnimator(scene_view, start_scene_coordinate),
		ZoneAnimator(zone_view, start_zone_coordinate, 0),
		RegionAnimator(region_view, start_region_coordinate)

{
	GroupPositioner = group_positioner;
	scene_view->addCameraAnimator(&SceneAnimator);

	using std::placeholders::_1;
	SceneAnimator.setMoveFinishEvent(std::bind(&RealmSceneView::moveFinished, scene_view, _1) );
	SceneAnimator.setMoveStartEvent(std::bind(&RealmSceneView::moveStarting, scene_view, _1) );
}

void RealmCharacterObjectPositioner::startTransition(ZoomState new_mode) {
	switch (new_mode) {

		case INTO_REGION:
		case OUT_REGION:
			SourceOffsetLocation = RegionAnimator.getPixelLocation();
			TargetOffsetLocation = RegionAnimator.getPixelLocation(true);
			break;

		case INTO_ZONE:
			SourceOffsetLocation = RegionAnimator.getPixelLocation();
			TargetOffsetLocation = ZoneAnimator.getPixelLocation(true);
			break;

		case OUT_ZONE:
			SourceOffsetLocation = ZoneAnimator.getPixelLocation();
			TargetOffsetLocation = RegionAnimator.getPixelLocation();
			break;

		case INTO_SCENE_PHASE1:
			SourceOffsetLocation = ZoneAnimator.getPixelLocation();
			TargetOffsetLocation = SceneAnimator.getPixelLocation(true);
			break;
		case INTO_SCENE_PHASE2:
			SourceOffsetLocation = SceneAnimator.getPixelLocation();
			TargetOffsetLocation = SceneAnimator.getPixelLocation(true);
			break;
		case OUT_SCENE_PHASE1:
			SourceOffsetLocation = SceneAnimator.getPixelLocation();
			TargetOffsetLocation = ZoneAnimator.getPixelLocation();
			break;
		case OUT_SCENE_PHASE2:
			SourceOffsetLocation = ZoneAnimator.getPixelLocation();
			TargetOffsetLocation = ZoneAnimator.getPixelLocation(true);
			break;
	}
}

void RealmCharacterObjectPositioner::update() {
	SceneAnimator.update();
	ZoneAnimator.update();
	RegionAnimator.update();
}


double RealmCharacterObjectPositioner::getScaleFactor() {

	double scale_factor = GroupPositioner->getScaleFactor();

	switch (GroupPositioner->getZoomState()) {
		case INTO_SCENE_PHASE2:
			return scale_factor + GroupPositioner->getZoomProgress();

		case OUT_SCENE_PHASE1:
			return scale_factor + 1 - GroupPositioner->getZoomProgress();

		case AT_SCENE:
			return scale_factor + 1;

		default:
			return scale_factor;
	}

}


XYLocation RealmCharacterObjectPositioner::getScreenLocation() {
	XYLocation CharacterGroupPosition = GroupPositioner->getScreenLocation();
	double zoom_progress = GroupPositioner->getZoomProgress();

	switch (GroupPositioner->getZoomState()) {
		case AT_REALM:
		case AT_REGION:
			return CharacterGroupPosition + RegionAnimator.getPixelLocation();

		case AT_ZONE:
			return CharacterGroupPosition + ZoneAnimator.getPixelLocation();

		case AT_SCENE:
		case INTO_SCENE_PHASE2:
			return CharacterGroupPosition + SceneAnimator.getPixelLocation();

		case OUT_SCENE_PHASE1:
		case INTO_REGION:
		case OUT_REGION:

		case INTO_ZONE:
		case OUT_ZONE:

		case INTO_SCENE_PHASE1:
		case OUT_SCENE_PHASE2:
			XYLocation CurrentOffsetLocation = XYLocation::findPointBetween(SourceOffsetLocation, TargetOffsetLocation, zoom_progress );
			return CharacterGroupPosition + CurrentOffsetLocation;
	}
}


