#include "RealmElevationTriangle.h"
#include "RealmSceneElevationSubMap.h"

RealmElevationTriangleSide::RealmElevationTriangleSide() {
}

void RealmElevationTriangleSide::init(RealmElevationTriangle* parent, char triangle_pos) {
	ParentTriangle = parent;
	PositionInTriangle = triangle_pos;
}

int RealmElevationTriangleSide::setSpace(int layer, int idx, int val, bool propagate_neighbors)
{
	int maxval =
		(layer == 0 ? 1 :
		(layer == 1 ? 4 :
		(layer == 2 ? 7 :
		(layer == 3 ? 10 :
		(layer == 4 ? 13 :
		-1)
		))));

	int newval = val;
	newval = ParentTriangle->setTriangleMapSpace(PositionInTriangle, layer, idx, val);

	if (propagate_neighbors && layer == 4) {
		bool is_corner = false;
		// if this is one of the corner spots via an "inner" side, reset idx to use outer edge side before propagation
		if (PositionInTriangle == 0 && idx == maxval) {
			idx = 0;
			is_corner = true;
		}
		if (PositionInTriangle == 2 && idx == 0) {
			idx = maxval;
			is_corner = true;
		}

		if (PositionInTriangle == 1 || is_corner) {
			RealmElevationTriangleSide* neighborside = ParentTriangle->getNeighborSide(1);
			newval = neighborside->setSpace(layer, maxval - idx, newval, false);
		}
	}

	return newval;
}

int RealmElevationTriangleSide::getSpace(int layer, int idx) {
	return ParentTriangle->getTriangleMapSpace(PositionInTriangle, layer, idx);
}


RealmElevationTriangle::RealmElevationTriangle() {
	NeighborSide[0] = nullptr;
	NeighborSide[1] = nullptr;
	NeighborSide[2] = nullptr;
}


void RealmElevationTriangle::init(RealmSceneElevationSubMap* parentmap, char pos, RealmNumberGenerator* num_generator) {
	ParentSceneMap = parentmap;
	PositionInParentMap = pos;
	CurrentPhase = 0;

	NumGenerator = num_generator;

	for (int i = 0; i < 3; i++) {
		TriangleSide[i].init(this, i);
	}
}

void RealmElevationTriangle::setNeighborSides(RealmElevationTriangleSide* neighborside1, RealmElevationTriangleSide* neighborside2, RealmElevationTriangleSide* neighborside3)
{
	NeighborSide[0] = neighborside1;
	NeighborSide[1] = neighborside2;
	NeighborSide[2] = neighborside3;
}

void RealmElevationTriangle::populatePhase(int phase) {
	int i;
	int influencers[5], weights[4];
	int result;
	int variance;
	if (phase == 1) {
		// calculates triangle center value, sets core spaces (layer 0) on each side
		int cornervalues[3];

		ParentSceneMap->populateToPhase(0);

		for (i = 0; i < 3; i++) {
			cornervalues[i] = TriangleSide[i].getSpace(4, 0);
		}

		variance = ParentSceneMap->getBaseVariance() * 5;
		centerValue = NumGenerator->calculateVarianceValue(variance, cornervalues, 3);

		int corecalc[2];

		variance = ParentSceneMap->getBaseVariance() * 2;

		// divide centerValue into 3 (+- generated offset weighted by distance from corners)
		for (i = 0; i < 3; i++) {
			corecalc[0] = TriangleSide[i].getSpace(4, 0);
			corecalc[1] = centerValue;

			// 1 / 13th the distance from center val to edge, so weight accordingly
			weights[0] = 1;
			weights[1] = 13;


			result = NumGenerator->calculateWeightedVarianceValue(variance, corecalc, weights, 2);

			// populate core array
			TriangleSide[i].setSpace(0, 0, result);
		}
	} else if (phase == 2) {

		int sidecorners[4];

		variance = ParentSceneMap->getBaseVariance() * 5;

		// calculate side centers
		for (i = 0; i < 3; i++) {
			// prereq: populate neighbor sides' parent triangle through phase 1
			// neighbors sides should all have their center values populated by now, but if not, make sure they have (todo)
			NeighborSide[i]->getParentTriangle()->populateToPhase(1);

			// int nexti = i < 2 ? i++ : 0;

			sidecorners[0] = TriangleSide[i].getSpace(4, 0);
			sidecorners[1] = centerValue;
			sidecorners[2] = TriangleSide[i].getSpace(4, 13);
			sidecorners[3] = NeighborSide[i]->getParentTriangle()->getCenterValue();

			weights[0] = 7;
			weights[1] = 11;
			weights[2] = 7;
			weights[3] = 11;


			result = NumGenerator->calculateWeightedVarianceValue(variance, sidecorners, weights, 4);
			TriangleSide[i].setCenterValue(result);

			// set the two center-adjacent side spaces
			int sideweight[2];
			sideweight[0] = 1;
			sideweight[1] = 13;

			int sidecalc[2];
			sidecalc[0] = TriangleSide[i].getSpace(4, 0);
			sidecalc[1] = TriangleSide[i].getCenterValue();
			result = NumGenerator->calculateWeightedVarianceValue(ParentSceneMap->getBaseVariance(), sidecalc, sideweight, 2);
			TriangleSide[i].setSpace(4, 6, result);

			sidecalc[0] = TriangleSide[i].getSpace(4, 13);
			sidecalc[1] = TriangleSide[i].getCenterValue();
			result = NumGenerator->calculateWeightedVarianceValue(ParentSceneMap->getBaseVariance(), sidecalc, sideweight, 2);
			TriangleSide[i].setSpace(4, 7, result);
		}
	} else if (phase == 3) {
		variance = ParentSceneMap->getBaseVariance() * 4;
		// calculate layer 2 corners ("3")
		for (i = 0; i < 3; i++) {
			int previ = i > 0 ? i-1 : 2;

			influencers[0] = TriangleSide[i].getSpace(4, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 6);
			influencers[2] = TriangleSide[i].getSpace(0, 0);
			influencers[3] = TriangleSide[previ].getSpace(4, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(2, 0, result);
		}
	} else if (phase == 4) {
		variance = ParentSceneMap->getBaseVariance() * 3;
		for (i = 0; i < 3; i++) {
			int previ = i > 0 ? i-1 : 2;
			// int nexti = i < 2 ? i++ : 0;

			// populate neighbors sides through phase 3
			NeighborSide[i]->getParentTriangle()->populateToPhase(3);

			weights[0] = 3;
			weights[1] = 2;
			weights[2] = 3;
			weights[3] = 2;

			influencers[0] = TriangleSide[i].getSpace(0, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 6);
			influencers[2] = TriangleSide[i].getSpace(2, 0);
			influencers[3] = TriangleSide[previ].getSpace(4, 7);
			result = NumGenerator->calculateWeightedVarianceValue(variance, influencers, weights, 4);
			TriangleSide[i].setSpace(1, 0, result);

			influencers[0] = TriangleSide[i].getSpace(2, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 6);
			influencers[2] = NeighborSide[i]->getSpace(2, 7);
			influencers[3] = TriangleSide[i].getSpace(4, 0);
			result = NumGenerator->calculateWeightedVarianceValue(variance, influencers, weights, 4);
			TriangleSide[i].setSpace(4, 3, result);

			influencers[0] = TriangleSide[i].getSpace(2, 7);
			influencers[1] = TriangleSide[i].getSpace(4, 13);
			influencers[2] = NeighborSide[i]->getSpace(2, 0);
			influencers[3] = TriangleSide[i].getSpace(4, 7);
			result = NumGenerator->calculateWeightedVarianceValue(variance, influencers, weights, 4);
			TriangleSide[i].setSpace(4, 10, result);
		}
	} else if (phase == 5) {
		variance = ParentSceneMap->getBaseVariance() * 3;
		for (i = 0; i < 3; i++) {
			int previ = i > 0 ? i-1 : 2;

			influencers[0] = TriangleSide[i].getSpace(4, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 3);
			influencers[2] = TriangleSide[i].getSpace(2, 0);
			influencers[3] = TriangleSide[previ].getSpace(4, 10);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(3, 0, result);

			influencers[0] = TriangleSide[i].getSpace(4, 6);
			influencers[1] = TriangleSide[i].getSpace(4, 3);
			// influencers[2] = TriangleSide[i].getSpace(2, 0);
			influencers[3] = TriangleSide[i].getSpace(1, 0);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(3, 3, result);

			influencers[0] = TriangleSide[i].getSpace(4, 10);
			influencers[1] = TriangleSide[i].getSpace(4, 7);
			influencers[2] = TriangleSide[i].getSpace(2, 7);
			influencers[3] = TriangleSide[i].getSpace(1, 4);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(3, 7, result);
		}
	} else if (phase == 6) {
		variance = ParentSceneMap->getBaseVariance() * 2;
		for (i = 0; i < 3; i++) {
			influencers[0] = TriangleSide[i].getSpace(0, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 6);
			influencers[2] = TriangleSide[i].getSpace(3, 3);
			influencers[3] = TriangleSide[i].getSpace(1, 0);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(2, 3, result);

			influencers[0] = TriangleSide[i].getSpace(4, 7);
			influencers[1] = TriangleSide[i].getSpace(0, 1);
			influencers[2] = TriangleSide[i].getSpace(1, 4);
			influencers[3] = TriangleSide[i].getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(2, 4, result);
		}
	} else if (phase == 7) {
		variance = ParentSceneMap->getBaseVariance() * 2;
		for (i = 0; i < 3; i++) {
			influencers[0] = TriangleSide[i].getSpace(0, 0);
			influencers[1] = TriangleSide[i].getSpace(2, 3);
			influencers[2] = TriangleSide[i].getSpace(1, 0);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(1, 1, result);

			influencers[0] = TriangleSide[i].getSpace(0, 0);
			influencers[1] = TriangleSide[i].getSpace(0, 1);
			influencers[2] = TriangleSide[i].getSpace(2, 3);
			influencers[3] = TriangleSide[i].getSpace(2, 4);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(1, 2, result);

			influencers[0] = TriangleSide[i].getSpace(0, 1);
			influencers[1] = TriangleSide[i].getSpace(1, 4);
			influencers[2] = TriangleSide[i].getSpace(2, 4);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(1, 3, result);


			influencers[0] = TriangleSide[i].getSpace(2, 0);
			influencers[1] = TriangleSide[i].getSpace(1, 0);
			influencers[2] = TriangleSide[i].getSpace(3, 3);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(2, 1, result);

			influencers[0] = TriangleSide[i].getSpace(2, 3);
			// redundant:
			//influencers[1] = TriangleSide[i].getSpace(1, 0);
			//influencers[2] = TriangleSide[i].getSpace(3, 3);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(2, 2, result);

			influencers[0] = TriangleSide[i].getSpace(2, 4);
			influencers[1] = TriangleSide[i].getSpace(1, 4);
			influencers[2] = TriangleSide[i].getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(2, 5, result);

			influencers[0] = TriangleSide[i].getSpace(2, 7);
			// redundant:
			//influencers[1] = TriangleSide[i].getSpace(1, 4);
			//influencers[2] = TriangleSide[i].getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(2, 6, result);


			influencers[0] = TriangleSide[i].getSpace(2, 0);
			influencers[1] = TriangleSide[i].getSpace(4, 3);
			influencers[2] = TriangleSide[i].getSpace(3, 0);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 1, result);

			// redundant:
			//influencers[0] = TriangleSide[i].getSpace(2, 0);
			//influencers[1] = TriangleSide[i].getSpace(4, 3);
			influencers[2] = TriangleSide[i].getSpace(3, 3);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 2, result);

			influencers[0] = TriangleSide[i].getSpace(2, 3);
			influencers[1] = TriangleSide[i].getSpace(4, 6);
			// redundant:
			//influencers[2] = TriangleSide[i].getSpace(3, 3);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 4, result);

			influencers[0] = TriangleSide[i].getSpace(2, 3);
			influencers[1] = TriangleSide[i].getSpace(2, 4);
			influencers[2] = TriangleSide[i].getSpace(4, 6);
			influencers[3] = TriangleSide[i].getSpace(4, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 4);
			TriangleSide[i].setSpace(3, 5, result);

			influencers[0] = TriangleSide[i].getSpace(4, 7);
			// redundant:
			//influencers[1] = TriangleSide[i].getSpace(2, 4);
			influencers[2] = TriangleSide[i].getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 6, result);

			influencers[0] = TriangleSide[i].getSpace(4, 10);
			influencers[1] = TriangleSide[i].getSpace(2, 7);
			// redundant:
			//influencers[2] = TriangleSide[i].getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 8, result);

			// redundant:
			//influencers[0] = TriangleSide[i].getSpace(4, 10);
			//influencers[1] = TriangleSide[i].getSpace(2, 7);
			influencers[2] = TriangleSide[i].getSpace(3, 10);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(3, 9, result);
		}
	} else if (phase == 8) {
		variance = ParentSceneMap->getBaseVariance() * 2;
		for (i = 0; i < 3; i++) {

			// populate neighbor side through phase 7
			NeighborSide[i]->getParentTriangle()->populateToPhase(7);

			influencers[0] = TriangleSide[i].getSpace(4, 0);
			influencers[1] = TriangleSide[i].getSpace(3, 0);
			influencers[2] = NeighborSide[i]->getSpace(3, 10);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(4, 1, result);

			influencers[0] = TriangleSide[i].getSpace(3, 0);
			influencers[1] = TriangleSide[i].getSpace(3, 1);
			influencers[2] = TriangleSide[i].getSpace(4, 3);
			influencers[3] = NeighborSide[i]->getSpace(3, 9);
			influencers[4] = NeighborSide[i]->getSpace(3, 10);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 2, result);

			influencers[0] = TriangleSide[i].getSpace(3, 2);
			influencers[1] = TriangleSide[i].getSpace(3, 3);
			// redundant:
			//influencers[2] = TriangleSide[i].getSpace(4, 3);
			influencers[3] = NeighborSide[i]->getSpace(3, 7);
			influencers[4] = NeighborSide[i]->getSpace(3, 8);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 4, result);

			influencers[0] = TriangleSide[i].getSpace(3, 3);
			influencers[1] = TriangleSide[i].getSpace(3, 4);
			influencers[2] = TriangleSide[i].getSpace(4, 6);
			influencers[3] = NeighborSide[i]->getSpace(3, 6);
			influencers[4] = NeighborSide[i]->getSpace(3, 7);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 5, result);

			influencers[0] = TriangleSide[i].getSpace(3, 6);
			influencers[1] = TriangleSide[i].getSpace(3, 7);
			influencers[2] = TriangleSide[i].getSpace(4, 7);
			influencers[3] = NeighborSide[i]->getSpace(3, 3);
			influencers[4] = NeighborSide[i]->getSpace(3, 4);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 8, result);

			influencers[0] = TriangleSide[i].getSpace(3, 7);
			influencers[1] = TriangleSide[i].getSpace(3, 8);
			influencers[2] = TriangleSide[i].getSpace(4, 10);
			influencers[3] = NeighborSide[i]->getSpace(3, 2);
			influencers[4] = NeighborSide[i]->getSpace(3, 3);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 9, result);

			influencers[0] = TriangleSide[i].getSpace(3, 9);
			influencers[1] = TriangleSide[i].getSpace(3, 10);
			// redundant:
			//influencers[2] = TriangleSide[i].getSpace(4, 10);
			influencers[3] = NeighborSide[i]->getSpace(3, 0);
			influencers[4] = NeighborSide[i]->getSpace(3, 1);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 5);
			TriangleSide[i].setSpace(4, 11, result);

			influencers[0] = TriangleSide[i].getSpace(4, 13);
			influencers[1] = TriangleSide[i].getSpace(3, 10);
			influencers[2] = NeighborSide[i]->getSpace(3, 0);
			result = NumGenerator->calculateVarianceValue(variance, influencers, 3);
			TriangleSide[i].setSpace(4, 12, result);
		}
	}
}

void RealmElevationTriangle::populateToPhase(int phase) {
	while (CurrentPhase <= phase) {
		populatePhase(CurrentPhase++);
	}
}

void RealmElevationTriangle::populate() {
	populateToPhase(8);
}


int RealmElevationTriangle::getMapIndexFromTrianglePosition(char sidepos, int layer, int index)
{
	int hexidx;

	int rdistance, phase, poffset;

	phase = PositionInParentMap;

	if (sidepos == 0) {
		// should produce same as commented out above, but simplified:
		rdistance = 8 - (layer * 2) + index;

		if (layer == 4) {
			phase = PositionInParentMap == 0 ? 5 : PositionInParentMap - 1;
			poffset = rdistance - 1;
		} else {
			poffset = 3 - layer;
		}

	} else if (sidepos == 1) {
		// should produce same result as commented out above, but simplified:
		rdistance = 9 + layer;
		if (layer == 4 && index == 0) {
			phase = PositionInParentMap > 0 ? PositionInParentMap - 1 : 5;
			poffset = 12;
		} else {
			poffset = index + (3 - layer);
		}

	} else if (sidepos == 2) {
		rdistance = 9 + layer - index;
		poffset = (4 + layer * 2) - index;
	}

	HexSpiralCoordinate coord(rdistance, phase, poffset);
	hexidx = coord.getHexIndex();
	return hexidx;
}


int RealmElevationTriangle::setTriangleMapSpace(char sidepos, int layer, int index, int val)
{
	int hexidx = getMapIndexFromTrianglePosition(sidepos, layer, index);

	return ParentSceneMap->setSpaceFromTriangle(hexidx, val);
}

int RealmElevationTriangle::getTriangleMapSpace(char sidepos, int layer, int index) {
	int hexidx = getMapIndexFromTrianglePosition(sidepos, layer, index);

	return ParentSceneMap->getElevation(hexidx);
}

