#ifndef __REALMMAPKEY_H
	#define __REALMMAPKEY_H

#include "RealmSceneKey.h"
#include "RealmLocation.h"
#include "const7.h"

struct RealmMapKey {
	RealmSceneKey MapCenterKey;

	// spiral coordinate, offset from the realm scene center referenced by above key
	int CenterSpiralOffset;

	RealmMapKey(const RealmLocation& l, int view_level): MapCenterKey(l, view_level) {
		// derive CenterSpiralOffset
		// based on which region of the parent map the spiral coordinate would lie within
		HexSpiralCoordinate mapcoord = l.getHexSpiralCoordinate(view_level);

		if (mapcoord.getHexIndex() < SCENE_SPIRAL_SIZE) {
			CenterSpiralOffset = 0;
		} else {
			CenterSpiralOffset = mapcoord
				.getParentSpiralCoordinate( !RealmLocation::isInvertedGrid(view_level) )
				.getHexIndex();
		}
	}

	RealmMapKey(const RealmMapKey& k): MapCenterKey(k.MapCenterKey) {
		CenterSpiralOffset = k.CenterSpiralOffset;
	}

	RealmMapKey(RealmSceneKey k, int centeroffset): MapCenterKey(k) {
		CenterSpiralOffset = centeroffset;
	}

	bool isInverted() const {
		return MapCenterKey.isInverted();
	}

	bool operator < (const RealmMapKey &ob) const
	{
		return MapCenterKey < ob.MapCenterKey ||
			( MapCenterKey == ob.MapCenterKey && CenterSpiralOffset < ob.CenterSpiralOffset );
	}
};


#endif // __REALMMAPKEY_H
