#include "RealmSceneBackground.h"
#include <SDL2/SDL.h>
#include "HexGridDisplay.h"
#include "HexSpiralCoordinate.h"
#include "RealmColorMap.h"
#include "RealmSceneElevationMap.h"
#include "RealmSceneView.h"

RealmSceneBackground::RealmSceneBackground(long int spiral_index, RealmSceneElevationMap* emap, int gridsize, bool inverted)
{
	Map = emap;
	HexRadius = 0;
	GridSize = gridsize;
	Inverted = inverted;

	HexGrid= new HexGridDisplay(inverted);
	HexGrid->setGridSize(gridsize);
	HexGrid->setLineColor(RGBColor(66, 77, 99));

	MapCoord = HexSpiralCoordinate(spiral_index)
				.getChildSpiralCoordinate(inverted)
				.getXY(inverted);

	ColorMap = new RealmColorMap(gridsize);

	emap->attachColorMap(ColorMap);
	HexGrid->attachColorMap(ColorMap);
	HexGrid->setFillEnabled(true);

	setSize(HexRadius);
}

RealmSceneBackground::~RealmSceneBackground() {

	Map->detachColorMap();

	delete HexGrid;
	delete ColorMap;
}

XYLocation RealmSceneBackground::getCenterCoordinate() {
	return XYLocation(Width/2, Height/2);
}

void RealmSceneBackground::setSize(int hex_radius) {
	HexGrid->setHexRadius(HexRadius = hex_radius);

	// set width and height
	if (!Inverted) {
		Height = (GridSize + 1) * 2 * SQRT_3 * HexRadius;
		Width = (GridSize + 1) * 2 * 1.5 * HexRadius;
	} else {
		Width = (GridSize + 1) * 2 * SQRT_3 * HexRadius;
		Height = (GridSize + 1) * 2 * 1.5 * HexRadius;
	}
}

bool RealmSceneBackground::needsUpdate() {
	return ColorMap->hasUpdated();
}

void RealmSceneBackground::update(RealmSceneView* scene_view) {

	int new_hex_radius = scene_view->getBaseRadius();
	if (HexRadius != new_hex_radius) {
		setSize(new_hex_radius);
		HexGrid->setCenter( getCenterCoordinate() );
	}

	ScreenLocation = scene_view->getScreenLocation(MapCoord);
}

void RealmSceneBackground::drawToTexture(SDL_Renderer* renderer, SDL_Texture* texture, bool force_update) {
	bool needs_update = false;

	if (ColorMap->hasUpdated() || force_update) {
		needs_update = true;
	}

	if (needs_update) {
		SDL_SetRenderTarget(renderer, texture);
		SDL_SetRenderDrawColor(renderer, 0x00, 0x00, 0x00, 0x00);
		SDL_RenderClear(renderer);

		HexGrid->setCenter( getCenterCoordinate() );
		HexGrid->draw(renderer);

		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_BLEND);
		SDL_SetRenderTarget(renderer, NULL);
	}
}
