#include "RealmSceneElevationMap.h"
#include "RealmColorMap.h"
#include "const7.h"
#include <assert.h>

RealmSceneElevationMap::RealmSceneElevationMap(RealmSceneElevationMapRepository* maprepo, RealmMapKey mapkey, int mapsize):
	MapKey(mapkey)
{
	ColorMap = nullptr;
	ElevationSpiral = new int[mapsize];
	MapSize = mapsize;
	MapRepository = maprepo;

	// -1 = blank
	for (int i = 0; i < MapSize; i++) {
		ElevationSpiral[i] = -1;
	}
}

RealmSceneElevationMap::~RealmSceneElevationMap() {
	delete [] ElevationSpiral;
}

void RealmSceneElevationMap::populateColorMap() {
	// populate color map with elevation
	for (int i = 0; i < MapSize; i++) {
		updateColorMapHex(i, true);
	}
}

void RealmSceneElevationMap::attachColorMap(RealmColorMap* cm) {
	ColorMap = cm;
	populateColorMap();
}

void RealmSceneElevationMap::detachColorMap() {
	ColorMap = nullptr;
}

void RealmSceneElevationMap::updateColorMapHex(int hexindex, bool first_display) {

	if (ColorMap) {
		int elev = getElevation(hexindex);
		int alpha = 255;
		int r=0, g=0, b=0;
		if (elev > 0) {
			if (elev < 5000) {
				// ocean waters
				int bottom_r = 15, bottom_g = 15, bottom_b = 36;
				int top_r = 20, top_g = 20, top_b = 216;

				int delta = elev; // should be between 0 and 5000

				r = bottom_r + (delta * (top_r - bottom_r) / 5000);
				g = bottom_g + (delta * (top_g - bottom_g) / 5000);
				b = bottom_b + (delta * (top_b - bottom_b) / 5000);

			} else if (elev < 10000) {
				// sandy beaches / dirt brown
				int bottom_r = 222, bottom_g = 200, bottom_b = 180;
				int top_r = 204, top_g = 135, top_b = 38;

				int delta = elev - 5000; // should be between 0 and 5000

				r = bottom_r + (delta * (top_r - bottom_r) / 5000);
				g = bottom_g + (delta * (top_g - bottom_g) / 5000);
				b = bottom_b + (delta * (top_b - bottom_b) / 5000);
			} else if (elev < 13000) {
				// green / yellowish grasses
				int bottom_r = 74, bottom_g = 240, bottom_b = 69;
				int top_r = 153, top_g = 222, top_b = 18;

				int delta = elev - 10000; // between 0 and 3000

				r = bottom_r + (delta * (top_r - bottom_r) / 3000);
				g = bottom_g + (delta * (top_g - bottom_g) / 3000);
				b = bottom_b + (delta * (top_b - bottom_b) / 3000);
			} else if (elev < 18000) {
				// deep green forests
				int bottom_r = 20, bottom_g = 200, bottom_b = 18;
				int top_r = 3, top_g = 80, top_b = 3;

				int delta = elev - 13000; // should be between 0 and 5000

				r = bottom_r + (delta * (top_r - bottom_r) / 5000);
				g = bottom_g + (delta * (top_g - bottom_g) / 5000);
				b = bottom_b + (delta * (top_b - bottom_b) / 5000);
			} else if (elev < 25000) {
				// mountain rocks to snow peaks
				int bottom_r = 188, bottom_g = 90, bottom_b = 194;
				int top_r = 188, top_g = 234, top_b = 232;

				int delta = elev - 18000; // between 0 and 7000 // 3000

				r = bottom_r + (delta * (top_r - bottom_r) / 7000);
				g = bottom_g + (delta * (top_g - bottom_g) / 7000);
				b = bottom_b + (delta * (top_b - bottom_b) / 7000);
			} else {
				// old default greyscale pattern
				r = g = b = elev * 255 / 24788;
			}
		} else {
			if (first_display) {
				// r = 50; g = 40; b = 25;
				r = g = b = 0;
				alpha = 0;
			} else {
				r = 180; g = 140; b = 120;
			}
		}
		ColorMap->setHex(hexindex, RGBColor(r, g, b, alpha));
	}
}

int RealmSceneElevationMap::setElevation(int hexindex, int value) {
	assert(hexindex < MapSize);

	// refuse to set if this space has already been populated, return original value
	if (ElevationSpiral[hexindex] != -1)
		return ElevationSpiral[hexindex];

	ElevationSpiral[hexindex] = value;
	updateColorMapHex(hexindex, false);

	// return new value
	return ElevationSpiral[hexindex];
}

int RealmSceneElevationMap::getElevation(int spiralidx) {
	assert(spiralidx < MapSize);
	return ElevationSpiral[spiralidx];
};


unsigned long RealmSceneElevationMap::getChildSeed(int spiralidx) {
	return NumGenerator.getStartSeed() + 8 + spiralidx * 2;
}

unsigned long RealmSceneElevationMap::getSubSeed(int spiralidx) {
	return NumGenerator.getStartSeed() + 9 + spiralidx;
}


