

#include "SceneMoveAnimator.h"
#include "RealmSceneView.h"

SceneMoveAnimator::SceneMoveAnimator(RealmSceneView* sceneview, HexSpiralCoordinate startCoordinate, int zoom_level): CurrentCoordinate(startCoordinate), TargetCoordinate(0)
{
	AnimatingMove = false;
	moveStartEvent = moveFinishEvent = nullptr;
	SceneView = sceneview;
	ZoomLevel = zoom_level;
	Inverted = (ZoomLevel % 2 != 0) ? SceneView->isInverted() : !SceneView->isInverted();

	setHexPosition(startCoordinate);

	for (int i = 0; i < 10; i++)
		ActiveMovementKeys[i] = false;
}

void SceneMoveAnimator::setZoomLevel(int l) {
	ZoomLevel = l;
	Inverted = (ZoomLevel % 2 != 0) ? SceneView->isInverted() : !SceneView->isInverted();
}

XYLocation SceneMoveAnimator::getGridPosition()
{
	return GridPosition;
}

void SceneMoveAnimator::setHexPosition(HexSpiralCoordinate newpos) {
	CurrentCoordinate = newpos;
	GridPosition = newpos.getXY(Inverted);
	AnimatingMove = false;

	int zoomradius = SceneView->getZoomRadius(ZoomLevel, true);
	double xscalefactor = zoomradius * SceneView->getXScaleFactor(ZoomLevel);
	double yscalefactor = zoomradius * SceneView->getYScaleFactor(ZoomLevel);

	PixelAnimator.setPosition(GridPosition.x * xscalefactor, GridPosition.y * yscalefactor);
	PixelAnimator.stop();
}

void SceneMoveAnimator::refreshPosition() {
	int zoomradius = SceneView->getZoomRadius(ZoomLevel, true);
	double xscalefactor = zoomradius * SceneView->getXScaleFactor(ZoomLevel);
	double yscalefactor = zoomradius * SceneView->getYScaleFactor(ZoomLevel);

	PixelAnimator.setPosition(GridPosition.x * xscalefactor, GridPosition.y * yscalefactor);
}

XYLocation SceneMoveAnimator::getCurrentOffset()
{
	if (AnimatingMove)
		return PixelAnimator.getOffset();
	else
		return XYLocation(0,0);
}

XYLocation SceneMoveAnimator::getScreenLocation(bool use_zoom_target) {
	return SceneView->getScreenLocation(getGridPosition(), use_zoom_target) + getCurrentOffset();
}

XYLocation SceneMoveAnimator::getPixelLocation(bool use_zoom_target) {

	int zoomradius = SceneView->getZoomRadius(ZoomLevel, use_zoom_target);
	double xscalefactor = zoomradius * SceneView->getXScaleFactor(ZoomLevel);
	double yscalefactor = zoomradius * SceneView->getYScaleFactor(ZoomLevel);

	XYLocation orig(GridPosition.getx() * xscalefactor, GridPosition.gety() * yscalefactor);
	return orig + PixelAnimator.getOffset();
}

double SceneMoveAnimator::getZoomScale() {
	return SceneView->getZoomScale(ZoomLevel);
}

// beginMovementKey to be called to start a movement action when a key is pressed and held
void SceneMoveAnimator::beginMovementKey(int key_direction) {
	ActiveMovementKeys[key_direction] = true;
}

// stopMovementKey is called when a key is released
void SceneMoveAnimator::stopMovementKey(int key_direction) {
	ActiveMovementKeys[key_direction] = false;
}

// todo: cleanup... these functions are a confusing mess
void SceneMoveAnimator::beginMove(HexSpiralCoordinate target) {
	TargetCoordinate = target;

	if (moveStartEvent)
		moveStartEvent(target);
	doMove(target.getXY(Inverted));
}

void SceneMoveAnimator::startMove(int direction) {
	TargetCoordinate = CurrentCoordinate.calcMovement(direction);
	if (moveStartEvent)
		moveStartEvent(TargetCoordinate);
	doMove(TargetCoordinate.getXY(Inverted));
}

void SceneMoveAnimator::updateMovementState() {
	int godir;

	if (AnimatingMove) return;

	if (ActiveMovementKeys[9]) {
		godir = Inverted ? 1 : 0;
		startMove(godir);
	} else if (ActiveMovementKeys[7]) {
		godir = 2;
		startMove(godir);
	} else if (ActiveMovementKeys[1]) {
		godir = Inverted ? 4 : 3;
		startMove(godir);
	} else if (ActiveMovementKeys[3]) {
		godir = 5;
		startMove(godir);
	} else if (ActiveMovementKeys[8]) {
		if (!Inverted) {
			godir = 1;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[2]) {
		if (!Inverted) {
			godir = 4;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[4]) {
		if (Inverted) {
			godir = 3;
			startMove(godir);
		}
	} else if (ActiveMovementKeys[6]) {
		if (Inverted) {
			godir = 0;
			startMove(godir);
		}
	}
}

void SceneMoveAnimator::doMove(XYLocation target)
{
	TargetPosition = target;
	AnimatingMove = true;

	int zoomradius = SceneView->getZoomRadius(ZoomLevel);
	double xscalefactor = zoomradius * SceneView->getXScaleFactor(ZoomLevel);
	double yscalefactor = zoomradius * SceneView->getYScaleFactor(ZoomLevel);

	PixelAnimator.doMove(XYLocation(TargetPosition.getx() * xscalefactor, TargetPosition.gety() * yscalefactor));
}

void SceneMoveAnimator::update() {
	updateMovementState();
	if (AnimatingMove) {
		if (PixelAnimator.advanceFrame()) {
			AnimatingMove = false;

			GridPosition = TargetPosition;
			CurrentCoordinate = TargetCoordinate;

			int zoomradius = SceneView->getZoomRadius(ZoomLevel, true);
			double xscalefactor = zoomradius * SceneView->getXScaleFactor(ZoomLevel);
			double yscalefactor = zoomradius * SceneView->getYScaleFactor(ZoomLevel);

			if (moveFinishEvent)
				moveFinishEvent(TargetCoordinate);
		}
	}
}
